searchNodes=[{"doc":"The NNTP client and server library.This module provides both the behaviour for an NNTP server, and the client API to interact with a NNTP server.ExampleThe GenNNTP behaviour abstracts the common NNTP client-server interaction. Developers are only required to implement the callbacks and functionality they are interested in to respond to client's commands, per specs.Let's start with a code example and then explore the available callbacks. Imagine we want a NNTP server that serves content from a directory.defmodule NServ do @behaviour GenNNTP # Callbacks @impl true def init(datadir) do {:ok, datadir} end @impl true def handle_CAPABILITIES(datadir) do {:ok, [&quot;READER&quot;], datadir} end @impl true def handle_HELP(datadir) do {:ok, &quot;This is some help text&quot;, datadir} end end # Start the server {:ok, pid} = GenNNTP.start(NServ, &quot;priv&quot;, port: 6791) # This is the client {:ok, socket} = GenNNTP.connect(&quot;localhost&quot;, 6791) {:ok, response} = GenNNTP.command(socket, &quot;CAPABILITIES&quot;) #=&gt; {:ok, &quot;101 Capability list:\\r\\nVERSION 2\\r\\nREADER&quot;}We start our NServ by calling start/3, passing the module with the server implementation and its initial argument (a path to a folder containing the data to serve from). We can primarily interact with the server by sending a command to the socket returned after connectiong to the server.Every time you do a GenNNTP.command/2, the client will send a command that must be handled by one of the callbacks defined in the GenNNTP, based on the issued command. There are many callbckas to be implemented when you use a GenNNTP. The required callbacks are init/1, handle_CAPABILITIES/1 and handle_HELP/1. Other callbacks are optional in the sense that they are still required to be implemented when your server has the capability for them. For example, if your server has &quot;READER&quot; capability, you MUST provide these callbacks: handle_GROUP/2, handle_LISTGROUP/2, handle_NEXT/2, handle_LAST/2, handle_ARTICLE/2, handle_HEAD/2, handle_BODY/2, handle_STAT/2.","ref":"GenNNTP.html","title":"GenNNTP","type":"behaviour"},{"doc":"Sends a command and receives server's response.","ref":"GenNNTP.html#command/3","title":"GenNNTP.command/3","type":"function"},{"doc":"Connects to a NNTP server.","ref":"GenNNTP.html#connect/3","title":"GenNNTP.connect/3","type":"function"},{"doc":"Invoked when a client selects an article.The arg is the argument used to specify the article to retrieve. It has 2 forms:A message-id.A 2-tuple containing the article number and the group name.Returning {:ok, { number, article }, new_state} sends the new current article number and the entire article to the client and continues the loop with new state new_state. A full article has message-id, the headers, and a body.Returning {:ok, false, new_state} sends 423 response to tell the client of unavailable article and continues the loop with new state new_state.Returning {:error, reason} to respond with reason and closes the client.Note that this callback is not invoked when currently selected newsgroup is invalid, or the current article number is invalid. GenNNTP handles those cases with appropriate response code. GenNNTP also handles the ARTICLE command with no argument, by taking the current article number instead before passing it to this callback.","ref":"GenNNTP.html#c:handle_ARTICLE/2","title":"GenNNTP.handle_ARTICLE/2","type":"callback"},{"doc":"Invoked when a client selects body of an article.This callback behaves identically to the handle_ARTICLE/2 except that only the body is returned in the article.","ref":"GenNNTP.html#c:handle_BODY/2","title":"GenNNTP.handle_BODY/2","type":"callback"},{"doc":"Invoked when a client asks for the server's capabilities.state is the current state of the NNTP server.The returning capabilities list is responded to the client, with &quot;VERSION&quot; always the first in the list. Server containues the loop with the new state.Only standard capabilities are responded to the client. Invalid ones in the callback's return are ignored.","ref":"GenNNTP.html#c:handle_CAPABILITIES/1","title":"GenNNTP.handle_CAPABILITIES/1","type":"callback"},{"doc":"Invoked when a client selects a newsgroup.group is the name of the newsgroup to be selected (e.g., &quot;news.software&quot;).Returning {:ok, group_summary, new_state} sends the group summary to client and continues the loop with new state new_state.Returning {:ok, false, new_state} sends 411 response to tell the client of unavailable grop and continues the loop with new state new_state.Returning {:error, reason} to respond with reason and closes the client.","ref":"GenNNTP.html#c:handle_GROUP/2","title":"GenNNTP.handle_GROUP/2","type":"callback"},{"doc":"Invoked when a client selects headers of an article.This callback behaves identically to the handle_ARTICLE/2 except that only the headers are returned in the article.","ref":"GenNNTP.html#c:handle_HEAD/2","title":"GenNNTP.handle_HEAD/2","type":"callback"},{"doc":"Invoked when a client wants summary of the server.Returning {:ok, help_test, new_state} to respond the help_text to the client and continues the loop with new state new_state.","ref":"GenNNTP.html#c:handle_HELP/1","title":"GenNNTP.handle_HELP/1","type":"callback"},{"doc":"Invoked when a client selects the previous article in the current newsgroup.The previous article in that newsgroup is the highest existing article number less than the current article number.Returning {:ok, { number, article }, new_state} sends the new current article number and the message-id of that article to the client and continues the loop with new state new_state.If number is the same as the current article number, a 422 is responded to tell the client of no next article in this group.Returning {:ok, false, new_state} sends 422 response to tell the client of unavailable article and continues the loop with new state new_state.Returning {:error, reason} to respond with reason and closes the client.Note that this callback is not invoked when currently selected newsgroup is invalid, or the current article number is invalid. GenNNTP handles those cases with appropriate response code.","ref":"GenNNTP.html#c:handle_LAST/2","title":"GenNNTP.handle_LAST/2","type":"callback"},{"doc":"Invoked when a client selects a newsgroup.group is the name of the newsgroup to be selected (e.g., &quot;news.software&quot;).Returning {:ok, group_summary, new_state} sends the group summary to client and continues the loop with new state new_state. The group summary is similar to the one responded by &quot;GROUP&quot; command, but also has a list of article numbers in the newsgroup.Returning {:ok, false, new_state} sends 411 response to tell the client of unavailable grop and continues the loop with new state new_state.Returning {:error, reason} to respond with reason and closes the client.","ref":"GenNNTP.html#c:handle_LISTGROUP/2","title":"GenNNTP.handle_LISTGROUP/2","type":"callback"},{"doc":"Invoked when a client selects the next article in the current newsgroup.The next article in that newsgroup is the lowest existing article number greater than the current article number.Returning {:ok, { number, article }, new_state} sends the new current article number and the message-id of that article to the client and continues the loop with new state new_state.If number is the same as the current article number, a 421 is responded to tell the client of no next article in this group.Returning {:ok, false, new_state} sends 421 response to tell the client of unavailable article and continues the loop with new state new_state.Returning {:error, reason} to respond with reason and closes the client.Note that this callback is not invoked when currently selected newsgroup is invalid, or the current article number is invalid. GenNNTP handles those cases with appropriate response code.","ref":"GenNNTP.html#c:handle_NEXT/2","title":"GenNNTP.handle_NEXT/2","type":"callback"},{"doc":"Invoked when a client sends an article to be posted.The callback receives a map of type article() without id field.Returning {:ok, new_state} to accept the article.Returning {:error, reason} to reject the article with specific reason.","ref":"GenNNTP.html#c:handle_POST/2","title":"GenNNTP.handle_POST/2","type":"callback"},{"doc":"Invoked when a client checks if an article exists.This callback behaves identically to the handle_ARTICLE/2 except that only the message-id is returned in the article.This callback allows the client to determine whether an article exists and in the second forms, what its message-id is, without having to process an arbitrary amount of text.","ref":"GenNNTP.html#c:handle_STAT/2","title":"GenNNTP.handle_STAT/2","type":"callback"},{"doc":"Invoked when an uknown command is asked by the client.This optional callback can be used to handle commands not understood by the current GenNNTP implementation. If not defined, GenNNTP responds with 500.The command is full command line sent by the client, minus the LFCR pair at the end. The callback can choose to reply or noreply to the command. The response does not need the ending LFCR pair.Returning {:error, reason} to respond with reason and closes the client.","ref":"GenNNTP.html#c:handle_command/2","title":"GenNNTP.handle_command/2","type":"callback"},{"doc":"Invoked when a client is connecting to the server.init_arg is the argument term (second argument) passed to start/3.Returning {:ok, state} wll start the handshake to establish the socket.Returning {:ok, state, timeout} is similar to {:ok, state}, except that it also sets a delay before establishing the handshake.Returning :ignore will make the process exit normally without entering the loop, closing the socket.Returning {:stop, reason} will cause the process to exit with reason reason without entering the loop, also closing the socket.","ref":"GenNNTP.html#c:init/1","title":"GenNNTP.init/1","type":"callback"},{"doc":"Starts a NNTP server with a callback module.Similar to starting a gen_server.","ref":"GenNNTP.html#start/3","title":"GenNNTP.start/3","type":"function"},{"doc":"Stops a NNTP server by its reference.The reference is usually the callback module.","ref":"GenNNTP.html#stop/1","title":"GenNNTP.stop/1","type":"function"},{"doc":"","ref":"GenNNTP.html#t:option/0","title":"GenNNTP.option/0","type":"type"}]